//*******************************************************************************
//* FILE NAME: arm_control.c
//*
//* DESCRIPTION:
//*	 Control of the 2007 three joint turreted arm
//*******************************************************************************

#include <stdio.h>
#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h" 
#include "arm_control.h"
#include "generic_menu.h"
#include "clock_routines.h"

 char arm_position;

//*******************************************************************************
//* FUNCTION NAME: arm_lookup_elbow
//* PURPOSE:       
//*	Lookup tables for proper relative arm joint positions
//*	As a time saver this is structured as a table indexed by a value of joint1 to the closest 10th.
//*	Corresponding joint2 values are then interpolated between the closest pair of joint1 ten's
//*	This just saves us from having to collect joint data for every possible position.
//* ARGUMENTS:
//*	position - HIGH, MEDIUM, LOW
//*	joint1 - where joint 1 currently is (or anticipated position to be based on OI command)
//*	joint2 - OUTPUT - The joint2 value corresponding to joint1 above
//*******************************************************************************
void arm_lookup_elbow(unsigned char position, unsigned int joint1, unsigned int *joint2)
{
	long temp_joint;

	// Interpolate between the two bounding values for all joints
	switch(position)
	{
		case HIGH:  // Tallest
			if (joint1 < JOINT1_HIGH_MIN)
			{
				*joint2 = JOINT2_HIGH_MIN; // Take the minimum value when out of range
			}
			else if (joint1 > JOINT1_HIGH_MAX)
			{
				*joint2 = JOINT2_HIGH_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint2 = (((long)(joint1-JOINT1_HIGH_MIN) * (JOINT2_HIGH_MAX - JOINT2_HIGH_MIN)) / (JOINT1_HIGH_MAX - JOINT1_HIGH_MIN))  + JOINT2_HIGH_MIN;  // Interpolate for joint2 value
			}
			break;

		case MEDIUM:
			if (joint1 < JOINT1_MEDIUM_MIN)
			{
				*joint2 = JOINT2_MEDIUM_MIN; // Take the minimum value when out of range
			}
			else if (joint1 > JOINT1_MEDIUM_MAX)
			{
				*joint2 = JOINT2_MEDIUM_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint2 = (((long)(joint1-JOINT1_MEDIUM_MIN) * (JOINT2_MEDIUM_MAX - JOINT2_MEDIUM_MIN)) / (JOINT1_MEDIUM_MAX - JOINT1_MEDIUM_MIN))  + JOINT2_MEDIUM_MIN;  // Interpolate for joint2 value
			}
			break;

		case LOW:
			if (joint1 < JOINT1_LOW_MIN)
			{
				*joint2 = JOINT2_LOW_MIN; // Take the minimum value when out of range
			}
			else if (joint1 > JOINT1_LOW_MAX)
			{
				*joint2 = JOINT2_LOW_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint2 = (((long)(joint1-JOINT1_LOW_MIN) * (JOINT2_LOW_MAX - JOINT2_LOW_MIN)) / (JOINT1_LOW_MAX - JOINT1_LOW_MIN))  + JOINT2_LOW_MIN;  // Interpolate for joint2 value
			}
			break;

		case FLOOR:	// Invader Zim  "so very short"
			if (joint1 < JOINT1_FLOOR_MIN)
			{
				*joint2 = JOINT2_FLOOR_MIN; // Take the minimum value when out of range
			}
			else if (joint1 > JOINT1_FLOOR_MAX)
			{
				*joint2 = JOINT2_FLOOR_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint2 =  (((long) (joint1-JOINT1_FLOOR_MIN) * (JOINT2_FLOOR_MAX - JOINT2_FLOOR_MIN)) / (JOINT1_FLOOR_MAX - JOINT1_FLOOR_MIN))  + JOINT2_FLOOR_MIN;  // Interpolate for joint2 value
			}
			break;

		case PICKUP:
			if (joint1 < JOINT1_PICKUP_MIN)
			{
				*joint2 = JOINT2_PICKUP_MIN; // Take the minimum value when out of range
			}
			else if (joint1 > JOINT1_PICKUP_MAX)
			{
				*joint2 = JOINT2_PICKUP_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint2 =  (((long) (joint1-JOINT1_PICKUP_MIN) * (JOINT2_PICKUP_MAX - JOINT2_PICKUP_MIN)) / (JOINT1_PICKUP_MAX - JOINT1_PICKUP_MIN))  + JOINT2_PICKUP_MIN;  // Interpolate for joint2 value
			}
			break;
	}
 }

//*******************************************************************************
//* FUNCTION NAME: arm_lookup_shoulder
//* PURPOSE:       
//*	The opposite of arm_lookup this one finds joint1 based on the joint2 position
//* ARGUMENTS:
//*	position - HIGH, MEDIUM, LOW
//*	joint2 - where joint 2 currently is (or anticipated position to be based on OI command)
//*	joint1 - OUTPUT - The joint1 value corresponding to join21 above
//*******************************************************************************
void arm_lookup_shoulder(unsigned char position, unsigned int *joint1, unsigned int joint2)
{
	// Interpolate between the two bounding values for all joints
	switch(position)
	{
		case HIGH:  // Tallest
			if (joint2 < JOINT2_HIGH_MIN)
			{
				*joint1 = JOINT1_HIGH_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_HIGH_MAX)
			{
				*joint1 = JOINT1_HIGH_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint1 = (((long)(joint2-JOINT2_HIGH_MIN) * (JOINT1_HIGH_MAX - JOINT1_HIGH_MIN)) / (JOINT2_HIGH_MAX - JOINT2_HIGH_MIN))  + JOINT1_HIGH_MIN;  // Interpolate for joint2 value
			}
			break;

		case MEDIUM:
			if (joint2 < JOINT2_MEDIUM_MIN)
			{
				*joint1 = JOINT1_MEDIUM_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_MEDIUM_MAX)
			{
				*joint1 = JOINT1_MEDIUM_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint1 = (((long)(joint2-JOINT2_MEDIUM_MIN) * (JOINT1_MEDIUM_MAX - JOINT1_MEDIUM_MIN)) / (JOINT2_MEDIUM_MAX - JOINT2_MEDIUM_MIN))  + JOINT1_MEDIUM_MIN;  // Interpolate for joint2 value
			}
			break;

		case LOW:
			if (joint2 < JOINT2_LOW_MIN)
			{
				*joint1 = JOINT1_LOW_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_LOW_MAX)
			{
				*joint1 = JOINT1_LOW_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint1 = (((long)(joint2-JOINT2_LOW_MIN) * (JOINT1_LOW_MAX - JOINT1_LOW_MIN)) / (JOINT2_LOW_MAX - JOINT2_LOW_MIN))  + JOINT1_LOW_MIN;  // Interpolate for joint2 value
			}
			break;

		case FLOOR:	// Invader Zim  "so very short"
			if (joint2 < JOINT2_FLOOR_MIN)
			{
				*joint1 = JOINT1_FLOOR_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_FLOOR_MAX)
			{
				*joint1 = JOINT1_FLOOR_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint1 =  (((long) (joint2-JOINT2_FLOOR_MIN) * (JOINT1_FLOOR_MAX - JOINT1_FLOOR_MIN)) / (JOINT2_FLOOR_MAX - JOINT2_FLOOR_MIN))  + JOINT1_FLOOR_MIN;  // Interpolate for joint2 value
			}
			break;

		case PICKUP:
			if (joint2 < JOINT2_PICKUP_MIN)
			{
				*joint1 = JOINT1_PICKUP_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_PICKUP_MAX)
			{
				*joint1 = JOINT1_PICKUP_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint1 =  (((long) (joint2-JOINT2_PICKUP_MIN) * (JOINT1_PICKUP_MAX - JOINT1_PICKUP_MIN)) / (JOINT2_PICKUP_MAX - JOINT2_PICKUP_MIN))  + JOINT1_PICKUP_MIN;  // Interpolate for joint2 value
			}
			break;
	}
 }

//*******************************************************************************
//* FUNCTION NAME: arm_lookup_wrist
//* PURPOSE:       
//*	The opposite of arm_lookup this one finds joint1 based on the joint2 position
//* ARGUMENTS:
//*	position - HIGH, MEDIUM, LOW
//*	joint2 - where joint 2 currently is (or anticipated position to be based on OI command)
//*	joint1 - OUTPUT - The joint1 value corresponding to join21 above
//*******************************************************************************
void arm_lookup_wrist(unsigned char position, unsigned int *joint3, unsigned int joint2)
{
	// Interpolate between the two bounding values for all joints
	switch(position)
	{
		case HIGH:  // Tallest
			if (joint2 < JOINT2_HIGH_MIN)
			{
				*joint3 = JOINT3_HIGH_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_HIGH_MAX)
			{
				*joint3 = JOINT3_HIGH_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint3 = (((long)(joint2-JOINT2_HIGH_MIN) * (JOINT3_HIGH_MAX - JOINT3_HIGH_MIN)) / (JOINT2_HIGH_MAX - JOINT2_HIGH_MIN))  + JOINT3_HIGH_MIN;  // Interpolate for joint2 value
			}
			break;

		case MEDIUM:
			if (joint2 < JOINT2_MEDIUM_MIN)
			{
				*joint3 = JOINT3_MEDIUM_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_MEDIUM_MAX)
			{
				*joint3 = JOINT3_MEDIUM_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint3 = (((long)(joint2-JOINT2_MEDIUM_MIN) * (JOINT3_MEDIUM_MAX - JOINT3_MEDIUM_MIN)) / (JOINT2_MEDIUM_MAX - JOINT2_MEDIUM_MIN))  + JOINT3_MEDIUM_MIN;  // Interpolate for joint2 value
			}
			break;

		case LOW:
			if (joint2 < JOINT2_LOW_MIN)
			{
				*joint3 = JOINT3_LOW_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_LOW_MAX)
			{
				*joint3 = JOINT3_LOW_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint3 = (((long)(joint2-JOINT2_LOW_MIN) * (JOINT3_LOW_MAX - JOINT3_LOW_MIN)) / (JOINT2_LOW_MAX - JOINT2_LOW_MIN))  + JOINT3_LOW_MIN;  // Interpolate for joint2 value
			}
			break;

		case FLOOR:	// Invader Zim  "so very short"
			if (joint2 < JOINT2_FLOOR_MIN)
			{
				*joint3 = JOINT3_FLOOR_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_FLOOR_MAX)
			{
				*joint3 = JOINT3_FLOOR_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint3 =  (((long) (joint2-JOINT2_FLOOR_MIN) * (JOINT3_FLOOR_MAX - JOINT3_FLOOR_MIN)) / (JOINT2_FLOOR_MAX - JOINT2_FLOOR_MIN))  + JOINT3_FLOOR_MIN;  // Interpolate for joint2 value
			}
			break;

		case PICKUP:
			if (joint2 < JOINT2_PICKUP_MIN)
			{
				*joint3 = JOINT3_PICKUP_MIN; // Take the minimum value when out of range
			}
			else if (joint2 > JOINT2_PICKUP_MAX)
			{
				*joint3 = JOINT3_PICKUP_MAX; // Take the minimum value when out of range
			}
			else
			{
				*joint3 =  (((long) (joint2-JOINT2_PICKUP_MIN) * (JOINT3_PICKUP_MAX - JOINT3_PICKUP_MIN)) / (JOINT2_PICKUP_MAX - JOINT2_PICKUP_MIN))  + JOINT3_PICKUP_MIN;  // Interpolate for joint2 value
			}
			break;
	}
 }

//*******************************************************************************
//* FUNCTION NAME: arm_stowed
//* PURPOSE:       
//*	Move the arm to it's home starting position
//*******************************************************************************
#define TURRENT_DEADBAND 5
void arm_stowed(unsigned int *desired_joint1, unsigned int *desired_joint2, unsigned int *desired_joint3, unsigned int *desired_joint4)
{
	unsigned int joint1, joint2, joint3, joint4;

	// Current joint positions
	joint1 = ARM_SHOULDER_POT;
	joint2 = ARM_ELBOW_POT;
	joint3 = ARM_WRIST_POT;
	joint4 = TURRET_POT;

	// Do one (or more) joints in a particular order

	// Start by moving Wrist to Stowed position
	*desired_joint3 = WRIST_STOWED;
	*desired_joint4 = TURRET_STOWED;

	if (joint2 < ELBOW_90DEGREES)
	{	// Can start to move shoulder into final position
		*desired_joint2 = ELBOW_STOWED;
		if ((joint4 > TURRET_STOWED-TURRENT_DEADBAND) && 
			(joint4 < TURRET_STOWED+TURRENT_DEADBAND))
				*desired_joint1 = SHOULDER_STOWED;
	}
	else 
	{
		*desired_joint1 = JOINT1_MAX;  // Shoulder max
		*desired_joint2 = JOINT2_MAX;	// Tuck Elbow in as we swing the big arm up 
		if (joint1 > SHOULDER_STOW1)  // Have to raise arm to safely swing elbow around
		{
			*desired_joint2 = ELBOW_STOWED;	// Start putting the elbow away
		}
	}
}


//*******************************************************************************
//* FUNCTION NAME: arm_unfold
//* PURPOSE:       
//*	Move the arm to it's operational starting position, probably the high position
//*******************************************************************************
void arm_unfold(unsigned int *desired_joint1, unsigned int *desired_joint2, unsigned int *desired_joint3, unsigned int *desired_joint4)
{
	unsigned int joint1, joint2, joint3, joint4;

	// Current joint positions
	joint1 = ARM_SHOULDER_POT;
	joint2 = ARM_ELBOW_POT;
	joint3 = ARM_WRIST_POT;
	joint4 = TURRET_POT;

	// Do one (or more) joints in a particular order

	// Start by moving the Shoulder to open position
	*desired_joint4 = TURRET_UNFOLD_STATE1; // Keep turret in position
	*desired_joint1 = JOINT1_UNFOLD_STATE2; // Get shoulder moving

	if (joint1 > SHOULDER_90DEGREES)
	{	// Can start to move elbow into final position when we pass the halfway point
		*desired_joint2 = JOINT2_UNFOLD_STATE2;
		*desired_joint3 = JOINT3_UNFOLD_STATE2;
	}
	else 
	{
		*desired_joint2 = joint2;	// Tuck Elbow in as we swing the big arm up 
		*desired_joint3 = joint3;  // Shoulder max
	}
}


//*******************************************************************************
//* FUNCTION NAME: arm_humanload
//* PURPOSE:       
//*	Move the arm to human loading position
//*******************************************************************************
void arm_humanload(unsigned int *desired_joint1, unsigned int *desired_joint2, unsigned int *desired_joint3, unsigned int *desired_joint4)
{
	unsigned int joint1, joint2, joint3, joint4;

	// Current joint positions
	joint1 = ARM_SHOULDER_POT;
	joint2 = ARM_ELBOW_POT;
	joint3 = ARM_WRIST_POT;
	joint4 = TURRET_POT;

	// Do one (or more) joints in a particular order

	TUBE_DROP;	// Prepare to take on a tube
	*desired_joint4 = joint4;

	if (joint1 > SHOULDER_HUMANLOAD1)
	{	// Now move joints into final position
		*desired_joint2 = ELBOW_HUMANLOAD;
		*desired_joint3 = WRIST_HUMANLOAD;
		if (joint2 > ELBOW_HUMANLOAD1)	// Elbow is still in the way
		{
			*desired_joint1 = JOINT1_MAX; // Delay moving the Shoulder down
		}
	}
	else 
	{	// First fold the arm segments up out of the way
		*desired_joint1 = JOINT1_MAX;  // Shoulder max
		*desired_joint2 = JOINT2_MAX;	// Tuck Elbow in as we swing the big arm up 
		*desired_joint3 = JOINT3_MAX;	// Tuck Wrist in too
	}
}


//*******************************************************************************
//* FUNCTION NAME: arm_movement
//* PURPOSE:       
//*	Move the arm according to the requested joint positions
//*	Limit is 48" in short direction and 71" in the diagonal
//*  Motors: Turret CCW=255, Shoulder up=255, Elbow unfold=255, Wrist in=255
//*  Pots: Turret 255->max to min, Shoulder 255->min to max, Elbow 255->min to max, Wrist->min to max
//*******************************************************************************
#define JOINT_DEADBAND 	2	// 2 works
#define WRIST_DEADBAND 	3	// 5 sometimes works
#define TURRET_DEADBAND 	3	// 5 sometimes works
#define ELBOW_STALL_LIMIT 	20	//In slow loop units
#define ELBOW_STALL_COOLOFF	80	//In slow loop units
#define WRIST_STALL_LIMIT 	20	//In slow loop units
#define WRIST_STALL_COOLOFF	80	//In slow loop units
#define MAX_JOINT1_SPEED 	127	// was 127
#define MAX_JOINT2_SPEED 	127	// was 127
#define MAX_JOINT3_SPEED  	64	// was 64
#define MAX_JOINT4_SPEED  	100	// was 64
#define MIN_JOINT1_SPEED 	20	// was 20 (not noticeably used, kp1 overrides)
#define MIN_JOINT2_SPEED 	17	// was 17
#define MIN_JOINT3_SPEED  	17	// was 17
#define MIN_JOINT4_SPEED  	10	// was 10

void arm_movement(unsigned int desired_joint1, unsigned int desired_joint2, unsigned int desired_joint3, unsigned int desired_joint4)
{
	static unsigned int elbow_stall_counter=0, elbow_stall_cooldown=0;
	static unsigned int wrist_stall_counter=0, wrist_stall_cooldown=0;
	unsigned int joint1, joint2, joint3, joint4;
	unsigned int kp1, kp2, kp3, kp4;
	long motor;

	if ((arm_position==HIGH || arm_position==MEDIUM || arm_position==LOW) &&  (ARM_DUNK==1) )
	{
		desired_joint1 -= DUNK;
	}

	Coefficient_Input(&kp1, &kp2, &kp3, &kp4);

	//Current joint positions
	joint1 = ARM_SHOULDER_POT;
	joint2 = ARM_ELBOW_POT;
	joint3 = ARM_WRIST_POT;
	joint4 = TURRET_POT;

	// ************* Joint 1
	if (joint1 < (desired_joint1 - JOINT_DEADBAND))
	{
		motor = (long)(desired_joint1-joint1)* kp1/100;
		if(motor > MAX_JOINT1_SPEED) 
		{
			motor = MAX_JOINT1_SPEED;
		}
		else if (motor < MIN_JOINT1_SPEED)
		{
			motor = 0;
		}
		SHOULDER_MOTOR_1 = SHOULDER_MOTOR_2 = 127 + motor;
 	}
	else if(joint1 > (desired_joint1 + JOINT_DEADBAND))
	{
		motor = (long)(joint1-desired_joint1)* kp1/100;
		if(motor > MAX_JOINT1_SPEED) 
		{
			motor = MAX_JOINT1_SPEED;
		}
		else if (motor < MIN_JOINT1_SPEED)
		{
			motor = 0;
		}
		SHOULDER_MOTOR_1 = SHOULDER_MOTOR_2 = 127 - motor;
	}
	else
	{
		SHOULDER_MOTOR_1 = SHOULDER_MOTOR_2 = 127;
	}

	// ************* Joint 2
	if (joint2 < (desired_joint2 - JOINT_DEADBAND))
	{
		motor = (long)(desired_joint2-joint2)* kp2/100;
		if(motor > MAX_JOINT2_SPEED) 
		{
			motor = MAX_JOINT2_SPEED;
		}
		else if (motor < MIN_JOINT2_SPEED)
		{
			motor = 0;
		}
		motor = 127 + motor;
	}
	else if(joint2 > (desired_joint2 + JOINT_DEADBAND))
	{
		motor = (long)(joint2-desired_joint2)* kp2/100;
		if(motor > MAX_JOINT2_SPEED)
		{
			motor = MAX_JOINT2_SPEED;
		}
		else if (motor < MIN_JOINT2_SPEED)
		{
			motor = 0;
		}
		motor = 127 - motor;
	}
	else
	{
		motor = 127;
	}
	ELBOW_MOTOR = (unsigned char) motor;

	// ************* Joint 3
	if (joint3 < (desired_joint3 - WRIST_DEADBAND))
	{
		motor = (long)(desired_joint3-joint3)* kp3/100;
		if(motor > MAX_JOINT3_SPEED) 
		{
			motor = MAX_JOINT3_SPEED;
		}
		else if (motor < MIN_JOINT3_SPEED)
		{
			motor = 0;
		}
		motor = 127 + motor;
	}
	else if(joint3 > (desired_joint3 + WRIST_DEADBAND))
	{
		motor = (long)(joint3-desired_joint3)* kp3/100;
		if(motor > MAX_JOINT3_SPEED)
		{
			motor = MAX_JOINT3_SPEED;
		}
		else if (motor < MIN_JOINT3_SPEED)
		{
			motor = 0;
		}
		motor = 127 - motor;
	}
	else
	{
		motor = 127;
	}
	WRIST_MOTOR = (unsigned char) motor;

	// ************* Joint 4
	if (joint4 < (desired_joint4 - TURRET_DEADBAND))
	{
		motor = (long)(desired_joint4-joint4)* kp4/100;
		if(motor > MAX_JOINT4_SPEED) 
		{
			motor = MAX_JOINT4_SPEED;
		}
		else if (motor < MIN_JOINT4_SPEED)
		{
			motor = 0;
		}
		motor = 127 + motor;
	}
	else if(joint4 > (desired_joint4 + TURRET_DEADBAND))
	{
		motor = (long)(joint4-desired_joint4)* kp4/100;
		if(motor > MAX_JOINT4_SPEED)
		{
			motor = MAX_JOINT4_SPEED;
		}
		else if (motor < MIN_JOINT4_SPEED)
		{
			motor = 0;
		}
		motor = 127 - motor;
	}
	else
	{
		motor = 127;
	}
	TURRET_MOTOR = (unsigned char) motor;

	// Safety override for limits of arm joints
	if (joint1 < JOINT1_MIN && SHOULDER_MOTOR_1 < 127) 
	{
		SHOULDER_MOTOR_1 = SHOULDER_MOTOR_2 = 127;
		//printf("WARN SHOULDER LIMITa\r");
	}
	else if ( joint1 > JOINT1_MAX && SHOULDER_MOTOR_1 > 127)
	{
		SHOULDER_MOTOR_1 = SHOULDER_MOTOR_2 = 127;
		//printf("WARN SHOULDER LIMITb\r");
	}

	if (joint2 < JOINT2_MIN && ELBOW_MOTOR < 127)
 	{
		ELBOW_MOTOR = 127;
		//printf("WARN ELBOW LIMITa\r");
	}
	else if (joint2 > JOINT2_MAX && ELBOW_MOTOR > 127) 
	{
		ELBOW_MOTOR = 127;
		//printf("WARN ELBOW LIMITb\r");
	}

	if (joint3 < JOINT3_MIN && WRIST_MOTOR < 127)
 	{
		WRIST_MOTOR = 127;
		//printf("WARN WRIST LIMITa\r");
	}
	else if (joint3 > JOINT3_MAX && WRIST_MOTOR > 127) 
	{
		WRIST_MOTOR = 127;
		//printf("WARN WRIST LIMITb\r");
	}

	// Limit check on the turret rotation
	if ((TURRET_POT < TURRET_MIN) && (TURRET_MOTOR <127))
	{
		TURRET_MOTOR = 127;
	}
	if ((TURRET_POT > TURRET_MAX) && (TURRET_MOTOR >127))
	{
		TURRET_MOTOR = 127;
	}
}


//*******************************************************************************
//* FUNCTION NAME: Arm_Control
//* PURPOSE:       Handles all arm joint movement
//* NOTES:   PWM_in1, etc.
//*******************************************************************************
void Arm_Control(void)
{
	unsigned int desired_joint1, desired_joint2, desired_joint3, desired_joint4;
	static char last_arm_position=HIGH;  // High, Medium, Low, Floor, Pickup off the floor
	static char arm_mode = MANUAL_MODE; //Manual, semi-auto, auto

	static unsigned int counter=0;
	unsigned int kp1, kp2, kp3, kp4;
	unsigned int joint1, joint2, joint3, joint4;

	// Drive the joints to particular positions according to the joystick input and user selected arm height

	// Select arm operating position
	if (ARM_HEIGHT >= 0 && ARM_HEIGHT < 10)
	{
		arm_position = HIGH;
	}
	if (ARM_HEIGHT >= 10 && ARM_HEIGHT < 40)
	{
		arm_position = MEDIUM;
	}
	if (ARM_HEIGHT >= 40 && ARM_HEIGHT < 100)
	{
		arm_position = LOW;
	}
	if (ARM_HEIGHT >= 100 && ARM_HEIGHT < 170)
	{
		arm_position = FLOOR;
	}
	if (ARM_HEIGHT >= 170 && ARM_HEIGHT < 220)
	{
		arm_position = PICKUP;
	}
	if (ARM_HEIGHT >= 220 && ARM_HEIGHT < 255)
	{
		arm_position = NULL;
	}

	// Select secondary arm operating options
	if (ARM_MODE >= 0 && ARM_MODE < 10)
	{
		arm_mode = MANUAL_MODE;
	}
	if (ARM_MODE >= 120 && ARM_MODE < 140)
	{
		arm_mode = MANUAL_MODE;
		//arm_mode = HUMANLOAD_MODE;  // Used to be Pot Reset
	}
	if (ARM_MODE >= 240 && ARM_MODE < 255)
	{
		arm_mode = UNFOLD;
	}

	// Position Overrides
	if (arm_position==NULL) 
	{
		// Stow override
		if (STOW_ARM == 1)
			arm_position = STOWED;
		else if (arm_mode == UNFOLD)
			arm_position = UNFOLD;
		//else if (arm_mode == MANUAL_MODE)
		else
		{
			//printf("Menu1Trace\r");
			arm_position = MANUAL_MODE;
		}
		//else if (arm_mode == HUMANLOAD_MODE)
			//arm_position = HUMANLOAD_MODE;
	}

	if (arm_position != MANUAL_MODE)
	{
		Generic_Menu_Initialize = TRUE;
		//if(Clockds%20 == 0) printf("ARM_HEIGHT=%d\r",ARM_HEIGHT);
	}

	joint1 = ARM_SHOULDER_POT;
	joint2 = ARM_ELBOW_POT;
	joint3 = ARM_WRIST_POT;
	joint4 = TURRET_POT;

	switch (arm_position)
	{
		case NULL:   // Do Nothing
			SHOULDER_MOTOR_1 = 127; SHOULDER_MOTOR_2 = 127; 
			ELBOW_MOTOR = 127; WRIST_MOTOR = 127;
			TURRET_MOTOR = 254 - TURRET_DRIVE;  // Reverse the joystick so the direction is natural
			break;
		case STOWED:   // Arm in stowed position
			arm_stowed(&desired_joint1, &desired_joint2, &desired_joint3, &desired_joint4);
			arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);  
			break;
		case UNFOLD:   // Arm unfold from starting position
			arm_unfold(&desired_joint1, &desired_joint2, &desired_joint3, &desired_joint4);
			arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);  
			break;
		case HIGH:   // Arm in high position
			// Scale OI input to match joint1 pot range
			desired_joint4 = (long)TURRET_DRIVE * (TURRET_MAX - TURRET_MIN) / 255 + TURRET_MIN;
			desired_joint2 = (long)ARM_EXTENSION * (JOINT2_HIGH_MAX - JOINT2_HIGH_MIN) / 255 + JOINT2_HIGH_MIN;
			//arm_lookup_elbow(HIGH, desired_joint1, &desired_joint2);
			arm_lookup_shoulder(HIGH, &desired_joint1, joint2);
			arm_lookup_wrist(HIGH, &desired_joint3, joint2);
			arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4); // Set the joint motors
			//TURRET_MOTOR = 254 - TURRET_DRIVE;  // Reverse the joystick so the direction is natural
			break;
		case MEDIUM:   // Arm in medium position
			// Scale OI input to match joint1 pot range
			desired_joint4 = (long)TURRET_DRIVE * (TURRET_MAX - TURRET_MIN) / 255 + TURRET_MIN;
			desired_joint2 = (long)ARM_EXTENSION * (JOINT2_MEDIUM_MAX - JOINT2_MEDIUM_MIN) / 255 + JOINT2_MEDIUM_MIN;
			//arm_lookup_elbow(MEDIUM, desired_joint1, &desired_joint2);
			arm_lookup_shoulder(MEDIUM, &desired_joint1, joint2);
			arm_lookup_wrist(MEDIUM, &desired_joint3, joint2);
			arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4); //); // Set the joint motors
			//TURRET_MOTOR = 254 - TURRET_DRIVE;  // Reverse the joystick so the direction is natural
			break;
		case LOW:   // Arm in low position
			// Scale OI input to match joint1 pot range
			desired_joint4 = (long)TURRET_DRIVE * (TURRET_MAX - TURRET_MIN) / 255 + TURRET_MIN;
			desired_joint2 = (long)ARM_EXTENSION * (JOINT2_LOW_MAX - JOINT2_LOW_MIN) / 255 + JOINT2_LOW_MIN;
			//arm_lookup_elbow(LOW, desired_joint1, &desired_joint2);
			arm_lookup_shoulder(LOW, &desired_joint1, joint2);
			arm_lookup_wrist(LOW, &desired_joint3, joint2);
			arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);  // Set the joint motors
			//TURRET_MOTOR = 254 - TURRET_DRIVE;  // Reverse the joystick so the direction is natural
			break;
		case FLOOR:   // Arm in floor position -hovering ready for tube pickup
			// Scale OI input to match joint1 pot range
			desired_joint4 = (long)TURRET_DRIVE * (TURRET_MAX - TURRET_MIN) / 255 + TURRET_MIN;
            if (ARM_EXTENSION < 127) ARM_EXTENSION = 127; // Kludge to center the pickup position
			desired_joint2 = (long)ARM_EXTENSION * (JOINT2_FLOOR_MAX - JOINT2_FLOOR_MIN) / 255 + JOINT2_FLOOR_MIN;
			//arm_lookup_elbow(FLOOR, desired_joint1, &desired_joint2);
			arm_lookup_shoulder(FLOOR, &desired_joint1, joint2);
			arm_lookup_wrist(FLOOR, &desired_joint3, joint2);
			arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);  // Set the joint motors
			//TURRET_MOTOR = 254 - TURRET_DRIVE;  // Reverse the joystick so the direction is natural
			break;
		case PICKUP:   // pickup from floor
			desired_joint4 = (long)TURRET_DRIVE * (TURRET_MAX - TURRET_MIN) / 255 + TURRET_MIN;
            if (ARM_EXTENSION < 127) ARM_EXTENSION = 127; // Kludge to center the pickup position
			desired_joint2 = (long)ARM_EXTENSION * (JOINT2_PICKUP_MAX - JOINT2_PICKUP_MIN) / 255 + JOINT2_PICKUP_MIN;
			arm_lookup_shoulder(PICKUP, &desired_joint1, joint2);
			arm_lookup_wrist(PICKUP, &desired_joint3, joint2);
			arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);  // Set the joint motors
			//TURRET_MOTOR = 254 - TURRET_DRIVE;  // Reverse the joystick so the direction is natural
			break;
		//case POT_RESET: // Reset the joint pots to known positions
			//arm_movement(SHOULDER_90DEGREES, ELBOW_90DEGREES, WRIST_90DEGREES, TURRET_STOWED);  
			//TURRET_MOTOR = 254 - TURRET_DRIVE;  // Reverse the joystick so the direction is natural
			//break;
		case MANUAL_MODE:   // Manual position the joints
			//printf("Menu2Trace\r");
			Generic_Menu(&desired_joint1, &desired_joint2, &desired_joint3, &desired_joint4);	// Allow user to manually change each joint position
			arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);  
			break;
		//case HUMANLOAD_MODE: // Set the joint pots to the positions for loading from the human player
			//arm_humanload(&desired_joint1, &desired_joint2, &desired_joint3, &desired_joint4);
			//arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);  
			//arm_movement(SHOULDER_HUMANLOAD, ELBOW_HUMANLOAD, WRIST_HUMANLOAD, TURRET_STOWED);  
			//TURRET_MOTOR = 254 - TURRET_DRIVE;  // Reverse the joystick so the direction is natural
			//break;
		//case TEST:   // Whatever needs looking at
			//SHOULDER_MOTOR_1 = 127; SHOULDER_MOTOR_2 = 127; 
			//ELBOW_MOTOR = 127; WRIST_MOTOR = 127;
			//TURRET_MOTOR = 127;  // Reverse the joystick so the direction is natural
			//desired_joint4 = (long)TURRET_DRIVE * (TURRET_MAX - TURRET_MIN) / 255 + TURRET_MIN;
			//arm_movement(joint1, joint2, joint3, desired_joint4);  
			//break;
	}

	last_arm_position = arm_position;	// remember where we were last (Take care to stay within 72" box)

	// Limit check on the turret rotation
	if ((TURRET_POT < TURRET_MIN) && (TURRET_MOTOR <127))
	{
		TURRET_MOTOR = 127;
	}
	if ((TURRET_POT > TURRET_MAX) && (TURRET_MOTOR >127))
	{
		TURRET_MOTOR = 127;
	}

#if (1)
	counter++;
	if ((counter % 38) == 0)
	{
		//printf("arm_position=%d", arm_position);
		//printf("sonic_1=%d, sonic_2=%d \r", SONIC_1, SONIC_2);
		//printf("arm@%d ", arm_position);
		//printf("Tm=%d, Sm=%d,  Em=%d,  Wm=%d,", TURRET_MOTOR, SHOULDER_MOTOR_1, ELBOW_MOTOR, WRIST_MOTOR);
		//Coefficient_Input(&kp1, &kp2, &kp3, &kp4);
		//printf("kp4=%d, kp2=%d, kp3=%d, kp4=%d, ", kp4, kp1, kp2, kp3);
		//printf("\r");
		//printf("Tj=%d, Sj=%d, Ej=%d, Wj=%d, ", joint4, joint1, joint2, joint3);
		//if (arm_position < STOWED)
			//printf("dTj=%d, dSj=%d, dEj=%d, dWj=%d\r", desired_joint4, desired_joint1, desired_joint2, desired_joint3);
	}
#endif //MLM

} // End Arm_Control


//*******************************************************************************
//* FUNCTION NAME: Coefficient_Input
//* PURPOSE:       Inputs and scales pot inputs to use as changable coefficients
//* NOTES:   
//*******************************************************************************
void Coefficient_Input(unsigned int *kp1, unsigned int *kp2, unsigned int *kp3, unsigned int *kp4)
{
	// pot (0-255) maps directly into the kp range of (0-255)
	*kp1 = 1200;	// Shoulder - was 1200
	*kp2 = 163;		// Elbow - was 118
	*kp3 = 88;		// Wrist - was 66 & 34
	*kp4 = 500;		// Turret - was 100

	// Get coefficients
// Drop the last couple of bits that will be oscillating wildly anyway.
//	*kp1 = (Get_Analog_Value(rc_ana_in16)>>2) * 10; // returns a value of 10-2550 (0-255 was too small)
//	*kp2 = Get_Analog_Value(rc_ana_in15)>>2; // This gives us a value of 0-255
//	*kp3 = Get_Analog_Value(rc_ana_in14)>>2; // This gives us a value of 0-255 (good range for wrist)
//	*kp4 = Get_Analog_Value(rc_ana_in13)>>2; // This gives us a value of 0-255 (good range for wrist)
}
