/*******************************************************************************
* FILE NAME: auto_arm_routines.c
*
* DESCRIPTION:
*   Autonomous arm functions
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "stdio.h"

#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "arm_control.h"

/*******************************************************************************
* FUNCTION NAME: Arm_Auto_Unfold
* PURPOSE:       Unfold the arm from it's storage position
* ARGUMENTS:    none
*******************************************************************************/
void Arm_Auto_Unfold()
{
	unsigned int desired_joint1, desired_joint2, desired_joint3, desired_joint4;

	if (lastArmCmd != AUNFOLD)
	{
		printf("AUNFOLD 1\r");
		lastArmCmd = AUNFOLD;
	}

	Arm_Status = INCOMPLETE;
	arm_unfold(&desired_joint1, &desired_joint2, &desired_joint3, &desired_joint4);
	arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);

	if ((SHOULDER_MOTOR_1 == NEUTRAL) && (ELBOW_MOTOR==NEUTRAL) &&
		(WRIST_MOTOR==NEUTRAL) )	
	{ // All done (move on to next drive command)
		// if motors are all neutral then we must be done
		printf("AUNFOLD done\r");
		Arm_Status = COMPLETE;
		lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
	}
}


/*******************************************************************************
* FUNCTION NAME: Arm_Auto_High
* PURPOSE:      Move the arm to th eHigh scoring position
* ARGUMENTS:    none
*******************************************************************************/
void Arm_Auto_High()
{
	unsigned int desired_joint1, desired_joint2, desired_joint3, desired_joint4;
	unsigned int joint2;

	if (lastArmCmd != AHIGH)
	{
		printf("AHIGH 1\r");
		lastArmCmd = AHIGH;
	}

	Arm_Status = INCOMPLETE;
	desired_joint4 = TURRET_POT;
	joint2 = ARM_ELBOW_POT;
	desired_joint2 = (long)127 * (JOINT2_HIGH_MAX - JOINT2_HIGH_MIN) / 255 + JOINT2_HIGH_MIN;
	arm_lookup_shoulder(HIGH, &desired_joint1, joint2);
	arm_lookup_wrist(HIGH, &desired_joint3, joint2);
	arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4); // Set the joint motors

	if ((SHOULDER_MOTOR_1 == NEUTRAL) && (ELBOW_MOTOR==NEUTRAL) &&
		(WRIST_MOTOR==NEUTRAL) )	
	{ // All done (move on to next drive command)
		// if motors are all neutral then we must be done
		printf("AHIGH done\r");
		Arm_Status = COMPLETE;
		lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
	}
}

/*******************************************************************************
* FUNCTION NAME: Arm_Auto_Medium
* PURPOSE:      Move the arm to th eHigh scoring position
* ARGUMENTS:    none
*******************************************************************************/
void Arm_Auto_Medium()
{
	unsigned int desired_joint1, desired_joint2, desired_joint3, desired_joint4;
	unsigned int joint2;

	if (lastArmCmd != AMED)
	{
		printf("AMED 1\r");
		lastArmCmd = AMED;
	}

	Arm_Status = INCOMPLETE;

	desired_joint4 = TURRET_POT;
	desired_joint2 = (long)127 * (JOINT2_MEDIUM_MAX - JOINT2_MEDIUM_MIN) / 255 + JOINT2_MEDIUM_MIN;
	arm_lookup_shoulder(MEDIUM, &desired_joint1, joint2);
	arm_lookup_wrist(MEDIUM, &desired_joint3, joint2);
	arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4); // Set the joint motors

	if ((SHOULDER_MOTOR_1 == NEUTRAL) && (ELBOW_MOTOR==NEUTRAL) &&
		(WRIST_MOTOR==NEUTRAL) )	
	{ // All done (move on to next drive command)
		// if motors are all neutral then we must be done
		printf("AMED done\r");
		Arm_Status = COMPLETE;
		lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
	}
}


/*******************************************************************************
* FUNCTION NAME: Arm_Auto_Low
* PURPOSE:      Move the arm to th eHigh scoring position
* ARGUMENTS:    none
*******************************************************************************/
void Arm_Auto_Low()
{
	unsigned int desired_joint1, desired_joint2, desired_joint3, desired_joint4;
	unsigned int joint2;

	if (lastArmCmd != ALOW)
	{
		printf("ALOW 1\r");
		lastArmCmd = ALOW;
	}

	Arm_Status = INCOMPLETE;

	desired_joint4 = TURRET_POT;
	desired_joint2 = (long)127 * (JOINT2_LOW_MAX - JOINT2_LOW_MIN) / 255 + JOINT2_LOW_MIN;
	arm_lookup_shoulder(LOW, &desired_joint1, joint2);
	arm_lookup_wrist(LOW, &desired_joint3, joint2);
	arm_movement(desired_joint1, desired_joint2, desired_joint3, desired_joint4);  // Set the joint motors

	if ((SHOULDER_MOTOR_1 == NEUTRAL) && (ELBOW_MOTOR==NEUTRAL) &&
		(WRIST_MOTOR==NEUTRAL) )	
	{ // All done (move on to next drive command)
		// if motors are all neutral then we must be done
		printf("ALOW done\r");
		Arm_Status = COMPLETE;
		lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
	}
}


/*******************************************************************************
* FUNCTION NAME: Arm_Auto_Letgo
* PURPOSE:      Release tube
* ARGUMENTS:    none
*******************************************************************************/
void Arm_Auto_Letgo()
{
	printf("ALETGO 1\r");
	TUBE_DROP;
 	// All done (move on to next drive command)
	Arm_Status = COMPLETE;
	lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
}
void Arm_Auto_Letgo(void);

/*******************************************************************************
* FUNCTION NAME: Arm_Sonic
* PURPOSE:       Use sensor to search with arm for a spider
*				Do a search pattern by waving the arm back and forth from one extreme to another
*                Each function determines completion or failure.
* ARGUMENTS:     input: distance in raw ultrasonic units (11= close, 500 = far), speed 0-127
* Looking down at spider:	spider	Floor	Pipe	top of spider
*	(approx)				Low	76		38		17
*							24 to 34 seems to be a good look down range for seeing the spider
*******************************************************************************/
#define DEADBAND 5
#define SPEED_LIMIT 127
#define SETTLE 10

void Arm_Sonic(int distance, int speed)
{
  static char Delayed_Finish=0;
  int error;
  char init;

  if (lastArmCmd != ASONIC)
  {
    printf("ASONIC 1\r");
    init = TRUE; // Only sets the initial direction
    Arm_Search_Pattern(init);
    init = FALSE;
    lastArmCmd = ASONIC;
  }
	// First assume we're still going and set the motors to get there
	Arm_Status = INCOMPLETE;

	  // Limit the motor output so it doesn't whip around too fast & isn't > char
	Arm_Search_Pattern(init);

	error = distance - SONIC_1;
	printf("distance=%d, SONIC_1=%u, error=%d\r", distance, SONIC_1, error);
	if ((error <= DEADBAND) && (error >= -DEADBAND))
	{  
		//printf("********************************************\r");
		TURRET_MOTOR = NEUTRAL;
		Delayed_Finish++;
		if( (Delayed_Finish > SETTLE) ) 
		{ // We're there
			// All done (move on to next drive command)
			Arm_Auto_Letgo(); // Go ahead and drop the tube
			Arm_Status = COMPLETE;
    			lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
			TURRET_MOTOR = NEUTRAL;
			Delayed_Finish = 0;
    	 		printf("We're done\r");
		}
	}
	else
	{
			Delayed_Finish = 0; // Reset settle time to know we're really done
	}

	// Limit check on the turret rotation
	if ((TURRET_POT < TURRET_MIN) && (TURRET_MOTOR <NEUTRAL))
	{
		TURRET_MOTOR = NEUTRAL;
	}
	if ((TURRET_POT > TURRET_MAX) && (TURRET_MOTOR >NEUTRAL))
	{
		TURRET_MOTOR = NEUTRAL;
	}
}


/*******************************************************************************
* FUNCTION NAME: Arm_Search_Pattern
* PURPOSE:       Move the turret back and forth while the spider is not detected.
*******************************************************************************/
#define TURRET_SPEED_LIMIT 127
#define LEFT  1
#define RIGHT 0
void Arm_Search_Pattern(char init)
{
  static char direction=LEFT; // default
  static char full_search=FALSE;
  long error;

  if (init)
  { // Reset for a fresh search
 	if (start_position == STARTRIGHT)
		direction = LEFT;
	else
		direction = RIGHT;
  }

  if (direction == RIGHT)
  {
    if (TURRET_POT > (TURRET_MAX-20))
    {
      direction = LEFT; // Reverse direction
      TURRET_MOTOR = 0; // Stop motor for one cycle (abrupt but not quite so abrupt)
                      // May want to stretch this delay in reversing our engines
    }
    else // Keep going RIGHT
    {
      TURRET_MOTOR =  NEUTRAL + TURRET_SPEED_LIMIT;
    }
  }
  else // Direction is to the left
  {
    if (TURRET_POT < TURRET_MIN+20)
    {
      direction = RIGHT; // Reverse direction
      TURRET_MOTOR = 0; // Stop motor for one cycle (abrupt but not quite so abrupt)
                      // May want to stretch this delay in reversing our engines
    }
    else  // Keep going LEFT
    {
      TURRET_MOTOR =  NEUTRAL - TURRET_SPEED_LIMIT;
    }
  }
}



/*******************************************************************************
* FUNCTION NAME: Arm_Turret
* PURPOSE:      Position Turret
*******************************************************************************/
void Arm_Turret(int position)
{
	unsigned int joint1, joint2, joint3, desired_joint4;

	if (lastArmCmd != ATURRET)
	{
		printf("ATURRET 1\r");
		lastArmCmd = ATURRET;
	}

	Arm_Status = INCOMPLETE;
	desired_joint4 = position;
	joint1 = ARM_SHOULDER_POT;
	joint2 = ARM_ELBOW_POT;
	joint3 = ARM_WRIST_POT;
	arm_movement(joint1, joint2, joint3, desired_joint4);  // Set the joint motors

	if (TURRET_MOTOR == NEUTRAL)	
	{ // All done (move on to next drive command)
		printf("ATURRET done\r");
		Arm_Status = COMPLETE;
		lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
	}
}
