/*******************************************************************************
* FILE NAME: auto_drive_routines.c
*
* DESCRIPTION:
*   Autonomous drive functions
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "stdio.h"

#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"


/*******************************************************************************
* FUNCTION NAME: Drive_Sonic
* PURPOSE:       Use sensor to drive to a set distance from the rack (or an obstacle)
*                Each function determines completion or failure.
* ARGUMENTS:     input: distance in raw ultrasonic units (11= close, 500 = far), speed 0-127
* View of the spider:  Distance		IR		Sonic
*						0"			360		13
*						6"			600		13
*						7"			609		13 (reading limit)
*						12"			370		23
*						21"			230		40
*						24"			230		47
*						36"			130		71
*	The following readings taken with the sonic sensors positioned perfectly between upper & lower spiders and directly 
*	over the lower spider:
*											40-50 (optimum distance to rack for capping)
*											straight on to spider 40 = the distance (to the rest of the rack)
*											at 45 degree angle to spider 50 = the distance we want
* Looking down at spider:	spider	Floor	Pipe	top of spider
*	(approx)				Low	76		38		17
*							24 to 34 seems to be a good look down range for seeing the spider
*******************************************************************************/
#define KP     10/10   // Proportional gain
#define KI     0/100    // Integral gain (if a fraction then take care with MAXERROR)
#define DEADBAND 2
#define MAXERROR 300  // Maximum error we will allow to accumulate (MAXERROR < SPEED_LIMIT/KI)
#define SPEED_LIMIT 127
#define SETTLE 10

void Drive_Sonic(int distance, int speed)
{
  static char Delayed_Finish=0;
  static int cumerror=0;
  signed int error;
  signed int p, i;
  signed int output;


  if (lastDriveCmd != DSONIC)
  {
    printf("DSONIC 1\r");
    lastDriveCmd = DSONIC;
  }
	// First assume we're still going and set the motors to get there
	Drive_Status = INCOMPLETE;
	error = distance - SONIC_2;  // How far off the target are we?
	printf("distance=%d, SONIC_2=%u, error=%d\r", distance, SONIC_2, error);

	// If the spider is within hearing at any time then drop any tube we might be holding
	if (Spider_Sonic_Check() == TRUE)
	{
		TUBE_DROP;	// Drop any tube we might be holding
	}

	// Apply power Proportional to how far off we are
	p =  error * KP;

	// Integral - Correct if we've been off by a little bit or if it's taking us too long
	i = cumerror * KI;
	cumerror += error;
	if (cumerror > MAXERROR) cumerror = MAXERROR;
	if (cumerror < -MAXERROR) cumerror = -MAXERROR;

      output = p + i;  // Calculate for motor forward
	  // Limit the motor output so it doesn't whip around too fast & isn't > char

      if (output > SPEED_LIMIT) output = SPEED_LIMIT;
      if (output < -SPEED_LIMIT) output = -SPEED_LIMIT;
      LEFT_MOTOR_1 = NEUTRAL + (unsigned char) output;
      RIGHT_MOTOR_1 = NEUTRAL + (unsigned char) output; // The motors on one side are reversed

	if ((error <= DEADBAND) && (error >= -DEADBAND))
	{  
		//printf("********************************************\r");
		Delayed_Finish++;
		if( (Delayed_Finish > SETTLE) ) 
		{ // We're there
			// All done (move on to next drive command)
			Drive_Status = COMPLETE;
    			lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
			LEFT_MOTOR_1 = NEUTRAL;
			RIGHT_MOTOR_1 = NEUTRAL;
			Delayed_Finish = 0;
    	 		cumerror = 0;  // Can zero this to stop over-correcting
    	 		printf("We're done\r");
		}
	}
	else
	{
			Delayed_Finish = 0; // Reset settle time to know we're really done
	}
}


/*******************************************************************************
* FUNCTION NAME: Drive_Forward_Time
* PURPOSE:       No sensor substitute for driving straight for a given time at a given speed.
*                Each function determines completion or failure.
* ARGUMENTS:     input: time in tenths of seconds, speed 0-127
*******************************************************************************/
void Drive_Forward_Time(int time, int speed)
{
  static unsigned long duration; //in tenths of seconds

  if (lastDriveCmd != DFWD)
  {
    printf("DFWD 1\r");
    lastDriveCmd = DFWD;
    duration = Clockds + time; // Current time + duration = cmd completed
  }

  if (duration > Clockds)
  { // Keep going
    Drive_Status = INCOMPLETE;
    /* Might want to balance the motors here */
    LEFT_MOTOR_1 = NEUTRAL+speed;  //MLM Hartford
    RIGHT_MOTOR_1 = NEUTRAL+speed; 
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    LEFT_MOTOR_1 = NEUTRAL;
    RIGHT_MOTOR_1 = NEUTRAL;
	printf("DFWD done\r");
  }
}

/*******************************************************************************
* FUNCTION NAME: Drive_Backward_Time
* PURPOSE:       No sensor substitute for driving straight backwards
* ARGUMENTS:     input: time in tenths of seconds, speed 0-127
*******************************************************************************/
void Drive_Backward_Time(int time, int speed)
{
  static unsigned long duration; //in tenths of seconds

  if (lastDriveCmd != DBCK)
  {
    printf("DBCK 1\r");
    lastDriveCmd = DBCK;
    duration = Clockds + time; // Current time + duration = cmd completed
  }

  if (duration > Clockds)
  { // Keep going
    Drive_Status = INCOMPLETE;

    /* Might want to balance the motors here */
    LEFT_MOTOR_1 = NEUTRAL-speed;
    RIGHT_MOTOR_1 = NEUTRAL-speed;
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    LEFT_MOTOR_1 = NEUTRAL;
    RIGHT_MOTOR_1 = NEUTRAL;
  }
}



/*******************************************************************************
* FUNCTION NAME: Drive_Spin_In_Place_Time
* PURPOSE:       No sensor substitute for a gyro turn
* ARGUMENTS:     input: time in tenths of seconds, speed 0-127, direction clockwise or CCW
*******************************************************************************/
void Drive_Spin_In_Place_Time(int time, int speed, int direction)
{
  static unsigned long duration; //in tenths of seconds

  if (lastDriveCmd != DSPIN)
  {
    printf("DSPIN 1\r");
    lastDriveCmd = DSPIN;
    Drive_Status = INCOMPLETE;
    duration = Clockds + time; // Current time + duration = cmd completed
    PUSH_DOWN;   // Extend push-down pads
  }

  if (duration > Clockds)
  { // Keep going
    Drive_Status = INCOMPLETE;

    if (direction == CW) /* Spin Clockwise */
    {
      LEFT_MOTOR_1 = NEUTRAL+speed;
      RIGHT_MOTOR_1 = NEUTRAL-speed;

    }else /* Spin the opposite direction - Counter Clockwise */
    {
      LEFT_MOTOR_1 = NEUTRAL-speed;
      RIGHT_MOTOR_1 = NEUTRAL+speed;
    }
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    LEFT_MOTOR_1 = NEUTRAL;
    RIGHT_MOTOR_1 = NEUTRAL;
    PUSH_UP;  // Retract push-down pads
  }
}



/*******************************************************************************
* FUNCTION NAME: Drive_Stop
* PURPOSE:       Unnecessary since all commands end in a stop.
*                Each function determines completion or failure.
*******************************************************************************/
void Drive_Stop()
{
    printf("DSTOP 1\r");
  LEFT_MOTOR_1 = NEUTRAL;
  RIGHT_MOTOR_1 = NEUTRAL;

  // All done (move on to next drive command)
  Drive_Status = COMPLETE;
  lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
}


/*******************************************************************************
* FUNCTION NAME: Drive_Curve
* PURPOSE:       No sensor substitute for driving straight for a given time at a given speed.
*                Each function determines completion or failure.
* ARGUMENTS:     input: time in tenths of seconds,
*		Asumes a set speed of 127
*******************************************************************************/
void Drive_Curve(int time, int percent, int direction)
{
  static unsigned long duration; //in tenths of seconds

  if (lastDriveCmd != DCURVE)
  {
    printf("DCURVE 1\r");
    lastDriveCmd = DCURVE;
    duration = Clockds + time; // Current time + duration = cmd completed
  }

  if (duration > Clockds)
  { // Keep going
    Drive_Status = INCOMPLETE;
    /* Might want to balance the motors here */
    if (direction == CW)
    {
        LEFT_MOTOR_1 = NEUTRAL+FULLSPEED;
        RIGHT_MOTOR_1 = NEUTRAL+(FULLSPEED*percent/100); 
    }
    else
    {
        LEFT_MOTOR_1 = NEUTRAL+(FULLSPEED*percent/100);
        RIGHT_MOTOR_1 = NEUTRAL+FULLSPEED; 
    }
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    LEFT_MOTOR_1 = NEUTRAL;
    RIGHT_MOTOR_1 = NEUTRAL;
  }
}
