/*******************************************************************************
* FILE NAME: auto_misc_routines.c
*
* DESCRIPTION:
*  Autonomous miscellaneous functions
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"


/*******************************************************************************
* FUNCTION NAME: Delay
* PURPOSE:       Delay for a period of time in tenths of seconds
*                There is an implied SYNC here. This will pause all operations.
*******************************************************************************/
void Delay(int time)
{
  static unsigned long duration;
  static char startFlag;

  if (lastJointCmd != DELAY)
  {
    duration = (unsigned long) time;
    startFlag=TRUE;
    lastJointCmd = DELAY;
    Joint_Status = INCOMPLETE;
  }

  /* Don't start until both drive and Arm command queues are free */
  if (Drive_Status == COMPLETE) 
    Drive_Status = JOINT;
  if (Arm_Status   == COMPLETE) 
    Arm_Status   = JOINT;

  if ((Drive_Status == JOINT) & (Arm_Status == JOINT))
  {
    if (startFlag)
    {
      startFlag = FALSE;
      duration = Clockds + duration; // Current time + duration = cmd completed
    }

    if (duration > Clockds)
    { // Keep waiting
      Joint_Status = INCOMPLETE;
    }
    else
    { // All done (move on to next drive command)
      Joint_Status = COMPLETE;
      Drive_Status = COMPLETE;
      Arm_Status   = COMPLETE;
      lastJointCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    }
  }
}


/*******************************************************************************
* FUNCTION NAME: Synch_Drive_Arm
* PURPOSE:       Synchronize the Arm and Drive commands
*                Wait for both to COMPLETE, before proceeding
*******************************************************************************/
void Sync_Drive_Arm(int time)
{
  static unsigned long duration;
  static char startFlag;

  if (lastJointCmd != SYNC)
  {
    startFlag=TRUE;
    lastJointCmd = SYNC;
    Joint_Status = INCOMPLETE;
  }

  /* Don't start until both drive and Arm command queues are free */
  if (Drive_Status == COMPLETE) Drive_Status = JOINT;
  if (Arm_Status   == COMPLETE) Arm_Status   = JOINT;

  if ((Drive_Status == JOINT) & (Arm_Status == JOINT))
  {
    if (startFlag)
    {
      startFlag = FALSE;
      duration = Clockds + (unsigned long) time; // Delay to allow previous operations to settle
    }

    if (duration > Clockds)
    { // Delay to allow previous operations to settle
      Joint_Status = INCOMPLETE;
    }
    else
    {
      Joint_Status   = COMPLETE;
      Drive_Status   = COMPLETE;
      Arm_Status = COMPLETE;
    }
  }
}
