/*******************************************************************************
* FILE NAME: clock_routines.c
*
* DESCRIPTION:
*  This file contains the clock time keeping routines  
*  Uses system timer 3
*
* USAGE:
*  Does anybody really know what time it is?
*
*******************************************************************************/

#include <timers.h>
#include <stdio.h>

#include "ifi_aliases.h"
#include "ifi_default.h"

#define VARIABLE_DECLARE  /* causes global variables to be declared only once */
  #include "clock_routines.h"
#undef VARIABLE_DECLARE


/*******************************************************************************
* FUNCTION NAME: Initialize_Clock.c
*
* DESCRIPTION:
*  Initialize and start the internal timer for our clock
*  #ticks = elapsed_time * 10**7/prescale
*  65535 is maximum value of a 16-bit timer
*  255 is maximum value of an 8-bit timer
*  65535 - #ticks = timer preset to get overflow at desired time
*
* USAGE:
*  Called from User_Initialization()
*  e.g., 8-bit timer
*  note: set timer to (255 - ticks)
*  time(ms) = ticks prescale postscale
*   0.1        250     4         1
*   0.2        125    16         1 
*   0.4        250    16         1
*   1          125    16         5
*   2          125    16        10
*   2          250    16         5
*   3          125    16        15
*   4          250    16        10
*   6          250    16        15
*
*******************************************************************************/

void Initialize_Clock(void)
{
/* Use Timer3 (16-bits) to control our clock */
/* timer set to 4 ms */

  OpenTimer3(TIMER_INT_ON &
             T3_16BIT_RW &
             T3_SOURCE_INT &
             T3_PS_1_8
             );

  // Maximum for 16-bit 65535 - 5000 (4ms) = 60535
  WriteTimer3(CLOCKOFFSET);  /* Preload timer to overflow after 4ms */

  // FRC game specific initialization
  EndOfAuto=FALSE; // One second Flag warning of the pending end of Autonomous mode
  EndOfOpr=FALSE; // One second Flag warning of the pending end of Operator mode
}


/*******************************************************************************
* FUNCTION NAME: Update_Clock.c
*
* DESCRIPTION:
*  Retrieve the current time and update global time variables away from any possible interrupt
*  interference.
*******************************************************************************/
void Update_Clock(void)
{
    INTCONbits.GIEL = 0; /* Disable Low Priority Interrupts */
 	Clockms = MasterClockms; // Grab the time without risk of an interrupt corrupting it
    INTCONbits.GIEL = 1; /* Enable Low Priority Interrupts */

    Clockds = Clockms / 100;
    Clocksec = Clockms / 1000;

    if (Clocksec > LastSecOfAuto)
       EndOfAuto=TRUE; // One second Flag warning of the pending end of Autonomous mode
    if (Clocksec > LastSecOfOpr)
       EndOfOpr=TRUE; // One second Flag warning of the pending end of Operator mode

}
